using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Xml;

namespace synthesystems.MediaMVP.Radio
{
    class MediaMVPMain
    {
        [STAThread]
        public static int Main(string[] args)
        {
            try
            {
                if ((args.Length == 0) || (args[0].ToLower() != "/update"))
                {
                    Application.EnableVisualStyles();
                    Application.SetCompatibleTextRenderingDefault(false);
                    Application.Run(new frmGenre());
                }
                else
                {
                    MediaMVPMain.Update();
                }
            }
            catch (Exception ex)
            {
                System.Diagnostics.EventLog.WriteEntry(Application.ProductName, ex.Message, System.Diagnostics.EventLogEntryType.Error);
            }

            return 0;
        }

        public static void Update()
        {

            if (!Directory.Exists(Properties.Settings.Default.RadioFolder))
            {
                throw new System.ApplicationException("The Radio folder could not be found.");
            }
            else if (!Directory.Exists(Properties.Settings.Default.HauppageFolder))
            {
                throw new System.ApplicationException("The Hauppage program folder could not be found.");
            }
            else
            {
                const string strcBaseURL = "http://www.shoutcast.com/sbin/shoutcast-playlist.pls?rn={0}&file=filename.pls";
                DirectoryInfo difRadio;
                XmlDocument xdcRadioFolders = new XmlDocument();
                XmlNode nodGenreFolder;

                difRadio = new DirectoryInfo(Properties.Settings.Default.RadioFolder);
                xdcRadioFolders.Load(Path.Combine(Properties.Settings.Default.HauppageFolder, "Users\\Default\\Temp\\radi_favs.xml"));

                //Remove all non-readonly MVP files
                foreach (FileInfo filMVP in difRadio.GetFiles("*.MVP", SearchOption.AllDirectories))
                {
                    if ((filMVP.Attributes & FileAttributes.ReadOnly) != FileAttributes.ReadOnly) filMVP.Delete();
                }

                //Clear out empty subfolders in the radio folder, remove node from Hauppauge folder list
                foreach (DirectoryInfo difDelete in difRadio.GetDirectories())
                {
                    nodGenreFolder = xdcRadioFolders.SelectSingleNode(string.Format("/Root/FOLDER[PATH='{0}']", difDelete.FullName));
                    
                    if (nodGenreFolder != null) xdcRadioFolders.DocumentElement.RemoveChild(nodGenreFolder);
                    
                    if ((difDelete.GetFiles().Length == 0) && (difDelete.GetDirectories().Length == 0)) difDelete.Delete();
                }

                foreach (string strGenre in Properties.Settings.Default.SyncGenres)
                {
                    //Create the subfolder
                    DirectoryInfo difGenre = difRadio.CreateSubdirectory(strGenre);
                    XmlElement xelGenre = xdcRadioFolders.CreateElement("FOLDER"); // xdcRadioFolders.CreateElement("FOLDER");
                    XmlElement xelProperty;

                    if (xdcRadioFolders.SelectSingleNode(string.Format("/Root/FOLDER[PATH='{0}']", difGenre.FullName)) == null)
                    {
                        //Modify the Hauppage document to include this folder if it doesn't exis

                        xelProperty = xdcRadioFolders.CreateElement("TITLE");
                        xelProperty.InnerText = strGenre;
                        xelGenre.AppendChild(xelProperty);

                        xelProperty = xdcRadioFolders.CreateElement("PATH");
                        xelProperty.InnerText = difGenre.FullName;
                        xelGenre.AppendChild(xelProperty);

                        xelProperty = xdcRadioFolders.CreateElement("DATE");
                        xelProperty.InnerText = "1"; //This element is ignored so value is not important
                        xelGenre.AppendChild(xelProperty);

                        xelProperty = xdcRadioFolders.CreateElement("DATESTRING");
                        xelProperty.InnerText = string.Format("{0:dd/MM/yyyy HH:m}", DateTime.Now);
                        xelGenre.AppendChild(xelProperty);

                        xdcRadioFolders.DocumentElement.AppendChild(xelGenre);
                    }

                    //Get the radio station list from SHOUTcast
                    XmlDocument xdcStations = new XmlDocument();
                    xdcStations.Load(string.Format(Properties.Settings.Default.StationList, strGenre));


                    foreach (XmlNode nodStation in xdcStations.SelectNodes("/stationlist/station"))
                    {
                        if ((decimal.Parse(nodStation.Attributes["br"].Value) >= Properties.Settings.Default.MinBitrate) 
                            && (decimal.Parse(nodStation.Attributes["lc"].Value) >= Properties.Settings.Default.MinListeners)
                            && (nodStation.Attributes["mt"].Value == "audio/mpeg"))
                        {
                            //Create a file for each station, naming the file with the name of the station and content is the link
                            StringBuilder stbFileName = new StringBuilder(nodStation.Attributes["name"].Value);

                            int intPos = stbFileName.ToString().IndexOfAny(Path.GetInvalidFileNameChars());

                            while (intPos >= 0)
                            {
                                stbFileName.Remove(intPos, 1);
                                intPos = stbFileName.ToString().IndexOfAny(Path.GetInvalidFileNameChars());
                            }

                            stbFileName.Append(".MVP");

                            StreamWriter stmMVP = File.CreateText(Path.Combine(difGenre.FullName, stbFileName.ToString()));

                            stmMVP.Write(string.Format(strcBaseURL, nodStation.Attributes["id"].Value));

                            stmMVP.Close();
                        }
                    }
                }

                xdcRadioFolders.Save(Path.Combine(Properties.Settings.Default.HauppageFolder, "Users\\Default\\Temp\\radi_favs.xml"));
            }
        }
    }
}
