using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Xml;

namespace synthesystems.MediaMVP.Radio
{
    public partial class frmGenre : Form
    {
        private bool _boolChanged;

        public frmGenre()
        {
            InitializeComponent();
        }

        private void frmGenre_Load(object sender, EventArgs e)
        {
            try
            {
                Cursor.Current = Cursors.WaitCursor;
                //Get the GenreList from the SHOUTcast - use the setting in the application settings
                XmlDocument xdcGenre = new XmlDocument();

                xdcGenre.Load(Properties.Settings.Default.GenreList);

                foreach (XmlNode nod in xdcGenre.SelectNodes("/genrelist/genre"))
                {
                    lvwGenre.Items.Add(nod.Attributes["name"].Value);
                }
                foreach (string strGenre in Properties.Settings.Default.SyncGenres)
                {
                    ListViewItem lviCheck = lvwGenre.FindItemWithText(strGenre);
                    if (lviCheck != null) lviCheck.Checked = true;
                }
                lvwGenre.AutoResizeColumns(ColumnHeaderAutoResizeStyle.ColumnContent);
                _boolChanged = false;
            }
            catch (Exception ex)
            {
                System.Windows.Forms.MessageBox.Show(ex.ToString());
            }
            finally
            {
                Cursor.Current = Cursors.Default;
            }
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            try
            {
                Cursor.Current = Cursors.WaitCursor;
                System.Collections.Specialized.StringCollection strGenres = Properties.Settings.Default.SyncGenres;

                strGenres.Clear();
                foreach (ListViewItem lviGenre in lvwGenre.CheckedItems)
                {
                    strGenres.Add(lviGenre.Text);
                }

                //Other properties are bound to the settings on the form itself

                Properties.Settings.Default.Save();
                _boolChanged = false;
            }
            catch (Exception ex)
            {
                System.Windows.Forms.MessageBox.Show(ex.ToString());
            }
            finally
            {
                Cursor.Current = Cursors.Default;
            }
        }

        private void btnSelectAll_Click(object sender, EventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;
            foreach (ListViewItem lviGenre in lvwGenre.Items)
            {
                lviGenre.Checked = true;
            }
            Cursor.Current = Cursors.Default;
        }

        private void btnUnselectAll_Click(object sender, EventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;
            foreach (ListViewItem lviGenre in lvwGenre.Items)
            {
                lviGenre.Checked = false;
            }
            Cursor.Current = Cursors.Default;
        }

        private void btnUpdate_Click(object sender, EventArgs e)
        {
            try
            {
                Cursor.Current = Cursors.WaitCursor;
                if (_boolChanged) btnSave_Click(sender, e);
                MediaMVPMain.Update();
            }
            catch (Exception ex)
            {
                System.Windows.Forms.MessageBox.Show(ex.ToString());
            }
            finally
            {
                Cursor.Current = Cursors.Default;
            }       
        }

        private void frmGenre_FormClosing(object sender, FormClosingEventArgs e)
        {
            if ((_boolChanged) && (MessageBox.Show("Would you like to save your changes before exiting?",Application.ProductName, MessageBoxButtons.YesNo,MessageBoxIcon.Question) == DialogResult.Yes))
            {
                btnSave_Click(sender, e);
            }
        }

        private void lvwGenre_ItemChecked(object sender, ItemCheckedEventArgs e)
        {
            _boolChanged = true;
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void lvwGenre_ItemSelectionChanged(object sender, ListViewItemSelectionChangedEventArgs e)
        {
            if(e.IsSelected) e.Item.Checked = !e.Item.Checked;
        }

        private void btnRadioFolder_Click(object sender, EventArgs e)
        {
            fbdRadioFolder.SelectedPath = txtRadioFolder.Text;
            if (fbdRadioFolder.ShowDialog() == DialogResult.OK)
            {
                txtRadioFolder.Text = fbdRadioFolder.SelectedPath;
                _boolChanged = true;
            }
        }

        private void txtRadioFolder_TextChanged(object sender, EventArgs e)
        {
            _boolChanged = true;
        }

        private void nudMinBitrate_ValueChanged(object sender, EventArgs e)
        {
            _boolChanged = true;
        }

        private void btnHauppageFolder_Click(object sender, EventArgs e)
        {
            fbdRadioFolder.SelectedPath = txtHauppageFolder.Text;
            if (fbdRadioFolder.ShowDialog() == DialogResult.OK)
            {
                txtHauppageFolder.Text = fbdRadioFolder.SelectedPath;
                _boolChanged = true;
            }
        }

        private void txtHauppageFolder_TextChanged(object sender, EventArgs e)
        {
            _boolChanged = true;
        }

        private void numericUpDown1_ValueChanged(object sender, EventArgs e)
        {
            _boolChanged = true;
        }
    }
}